@@ -7,7 +7,6 @@ from datetime import datetime
 import pandas as pd
 from pytz import timezone
 
-from wetterdienst.metadata.provider import Provider
 from wetterdienst.metadata.timezone import Timezone
 
 
@@ -31,13 +30,6 @@ class Core(metaclass=ABCMeta):
         class, uses the Timezone enumeration"""
         pass
 
-    @property
-    @abstractmethod
-    def provider(self) -> Provider:
-        """Abstract representation of source that is related to the request, used for
-        identification of returned data"""
-        pass
-
     @property
     def _now_local(self) -> datetime:
         """Local now time based on the given timezone that represents the request time
