@@ -5,30 +5,26 @@ import logging
 from datetime import datetime
 from enum import Enum
 from io import StringIO
-from typing import Dict, Generator, Optional, Tuple, Union
+from typing import Dict, Generator, List, Optional, Tuple, Union
 from urllib.parse import urljoin
 
+import numpy as np
 import pandas as pd
 import pytz
 from requests import HTTPError
 
-from wetterdienst.core.scalar.request import ScalarRequestCore
-from wetterdienst.core.scalar.result import StationsResult, ValuesResult
-from wetterdienst.core.scalar.values import ScalarValuesCore
+from wetterdienst.core.timeseries.request import TimeseriesRequest
+from wetterdienst.core.timeseries.result import StationsResult, ValuesResult
+from wetterdienst.core.timeseries.values import TimeseriesValues
 from wetterdienst.exceptions import InvalidParameter
 from wetterdienst.metadata.columns import Columns
 from wetterdienst.metadata.datarange import DataRange
 from wetterdienst.metadata.kind import Kind
+from wetterdienst.metadata.parameter import Parameter
 from wetterdienst.metadata.period import Period, PeriodType
 from wetterdienst.metadata.provider import Provider
 from wetterdienst.metadata.resolution import Resolution, ResolutionType
 from wetterdienst.metadata.timezone import Timezone
-from wetterdienst.provider.dwd.metadata.column_names import DwdColumns
-from wetterdienst.provider.dwd.metadata.constants import (
-    DWD_MOSMIX_L_SINGLE_PATH,
-    DWD_MOSMIX_S_PATH,
-    DWD_SERVER,
-)
 from wetterdienst.provider.dwd.metadata.datetime import DatetimeFormat
 from wetterdienst.provider.dwd.mosmix.access import KMLReader
 from wetterdienst.provider.dwd.mosmix.metadata import (
@@ -37,6 +33,7 @@ from wetterdienst.provider.dwd.mosmix.metadata import (
     DwdMosmixType,
 )
 from wetterdienst.provider.dwd.mosmix.metadata.unit import DwdMosmixUnit
+from wetterdienst.settings import Settings
 from wetterdienst.util.cache import CacheExpiry
 from wetterdienst.util.enumeration import parse_enumeration_from_template
 from wetterdienst.util.geo import convert_dm_to_dd
@@ -44,13 +41,26 @@ from wetterdienst.util.network import download_file, list_remote_files_fsspec
 
 log = logging.getLogger(__name__)
 
+DWD_MOSMIX_S_PATH = "weather/local_forecasts/mos/MOSMIX_S/all_stations/kml/"
+DWD_MOSMIX_L_PATH = "weather/local_forecasts/mos/MOSMIX_L/all_stations/kml/"
+DWD_MOSMIX_L_SINGLE_PATH = "weather/local_forecasts/mos/MOSMIX_L/single_stations/{station_id}/kml/"
+
+
+class DwdMosmixPeriod(Enum):
+    FUTURE = Period.FUTURE.value
+
 
 class DwdMosmixDataset(Enum):
     SMALL = "small"
     LARGE = "large"
 
 
-class DwdMosmixValues(ScalarValuesCore):
+class DwdMosmixStationGroup(Enum):
+    SINGLE_STATIONS = "single_stations"
+    ALL_STATIONS = "all_stations"
+
+
+class DwdMosmixValues(TimeseriesValues):
     """
     Fetch weather mosmix data (KML/MOSMIX_S dataset).
 
@@ -70,10 +80,6 @@ class DwdMosmixValues(ScalarValuesCore):
     _data_tz = Timezone.UTC
     _has_quality = False
 
-    _irregular_parameters = ()
-    _string_parameters = ()
-    _date_parameters = ()
-
     def _create_humanized_parameters_mapping(self) -> Dict[str, str]:
         """
         Method for creation of parameter name mappings based on
@@ -83,7 +89,9 @@ class DwdMosmixValues(ScalarValuesCore):
         """
         return {
             parameter.value: parameter.name.lower()
-            for parameter in self.sr.stations._parameter_base[self.sr.stations.mosmix_type.name]
+            for parameter in self.sr.stations._parameter_base[self.sr.stations.mosmix_type.name][
+                self.sr.stations.mosmix_type.name
+            ]
         }
 
     def __init__(self, stations_result: StationsResult) -> None:
@@ -99,18 +107,15 @@ class DwdMosmixValues(ScalarValuesCore):
         parameter_ = []
         for parameter, dataset in self.sr.parameter:
             if parameter == dataset:
-                parameter = [par.value for par in parameter_base[dataset_accessor]]
+                parameter = [par.value for par in parameter_base[dataset_accessor][dataset_accessor]]
                 parameter_.extend(parameter)
             else:
                 parameter_.append(parameter.value)
 
         self.kml = KMLReader(
-            station_ids=self.sr.station_id.tolist(),
-            parameters=parameter_,
+            station_ids=self.sr.station_id.tolist(), parameters=parameter_, settings=self.sr.stations.settings
         )
 
-    # TODO: add __eq__ and __str__
-
     @property
     def metadata(self) -> pd.DataFrame:
         """
@@ -127,12 +132,13 @@ class DwdMosmixValues(ScalarValuesCore):
         :return:
         """
         hpm = self._create_humanized_parameters_mapping()
+        self.stations_counter = 0
+        self.stations_collected = []
 
         for df in self._collect_station_parameter():
-            df = self._coerce_parameter_types(df)
+            df = self._tidy_up_df(df, self.sr.stations.mosmix_type)
 
-            if self.sr.stations.tidy:
-                df = self.tidy_up_df(df, self.sr.stations.mosmix_type)
+            df = self._coerce_parameter_types(df)
 
             station_id = df[Columns.STATION_ID.value].iloc[0]
 
@@ -150,7 +156,16 @@ class DwdMosmixValues(ScalarValuesCore):
                     :,
                 ]
 
-            yield ValuesResult(stations=self.sr, df=df)
+            if not self.sr.tidy:
+                df = self._tabulate_df(df)
+
+            self.stations_counter += 1
+            self.stations_collected.append(station_id)
+
+            yield ValuesResult(stations=self.sr, values=self, df=df)
+
+            if self.stations_counter == self.sr.rank:
+                return
 
     def _collect_station_parameter(self) -> Generator[pd.DataFrame, None, None]:
         """
@@ -206,8 +221,7 @@ class DwdMosmixValues(ScalarValuesCore):
         for df_forecast in self._read_mosmix(date):
             df_forecast = df_forecast.rename(
                 columns={
-                    "station_id": DwdColumns.STATION_ID.value,
-                    "datetime": DwdColumns.DATE.value,
+                    "datetime": Columns.DATE.value,
                 }
             )
 
@@ -234,7 +248,7 @@ class DwdMosmixValues(ScalarValuesCore):
         :param date: datetime or enumeration for latest MOSMIX mosmix
         :return: pandas DataFrame with data
         """
-        url = urljoin(DWD_SERVER, DWD_MOSMIX_S_PATH)
+        url = urljoin("https://opendata.dwd.de", DWD_MOSMIX_S_PATH)
 
         file_url = self.get_url_for_date(url, date)
 
@@ -253,23 +267,30 @@ class DwdMosmixValues(ScalarValuesCore):
         :param date:
         :return:
         """
-        url = urljoin(DWD_SERVER, DWD_MOSMIX_L_SINGLE_PATH)
+        if self.sr.stations.station_group == DwdMosmixStationGroup.ALL_STATIONS:
+            url = urljoin("https://opendata.dwd.de", DWD_MOSMIX_L_PATH)
 
-        for station_id in self.sr.station_id:
-            station_url = f"{url}{station_id}/kml"
-
-            try:
-                file_url = self.get_url_for_date(station_url, date)
-            except HTTPError:
-                log.warning(f"Files for {station_id} do not exist on the server")
-                continue
+            file_url = self.get_url_for_date(url, date)
 
             self.kml.read(file_url)
 
-            yield next(self.kml.get_forecasts())
+            for forecast in self.kml.get_forecasts():
+                yield forecast
+        else:
+            for station_id in self.sr.station_id:
+                station_url = urljoin("https://opendata.dwd.de", DWD_MOSMIX_L_SINGLE_PATH).format(station_id=station_id)
 
-    @staticmethod
-    def get_url_for_date(url: str, date: Union[datetime, DwdForecastDate]) -> str:
+                try:
+                    file_url = self.get_url_for_date(station_url, date)
+                except HTTPError:
+                    log.warning(f"Files for {station_id} do not exist on the server")
+                    continue
+
+                self.kml.read(file_url)
+
+                yield next(self.kml.get_forecasts())
+
+    def get_url_for_date(self, url: str, date: Union[datetime, DwdForecastDate]) -> str:
         """
         Method to get a file url based on the MOSMIX-S/MOSMIX-L url and the date that is
         used for filtering.
@@ -278,7 +299,7 @@ class DwdMosmixValues(ScalarValuesCore):
         :param date: date used for filtering of the available files
         :return: file url based on the filtering
         """
-        urls = list_remote_files_fsspec(url, CacheExpiry.NO_CACHE)
+        urls = list_remote_files_fsspec(url, self.sr.stations.settings, CacheExpiry.NO_CACHE)
 
         if date == DwdForecastDate.LATEST:
             try:
@@ -291,17 +312,13 @@ class DwdMosmixValues(ScalarValuesCore):
 
         df_urls = pd.DataFrame({"URL": urls})
 
-        df_urls[DwdColumns.DATE.value] = df_urls["URL"].apply(
-            lambda url_: url_.split("/")[-1].split("_")[2].replace(".kmz", "")
-        )
+        df_urls["date"] = df_urls["URL"].apply(lambda url_: url_.split("/")[-1].split("_")[2].replace(".kmz", ""))
 
-        df_urls = df_urls[df_urls[DwdColumns.DATE.value] != "LATEST"]
+        df_urls = df_urls[df_urls["date"] != "LATEST"]
 
-        df_urls[DwdColumns.DATE.value] = pd.to_datetime(
-            df_urls[DwdColumns.DATE.value], format=DatetimeFormat.YMDH.value
-        )
+        df_urls["date"] = pd.to_datetime(df_urls["date"], format=DatetimeFormat.YMDH.value)
 
-        df_urls = df_urls.loc[df_urls[DwdColumns.DATE.value] == date]
+        df_urls = df_urls.loc[df_urls[Columns.DATE.value] == date]
 
         if df_urls.empty:
             raise IndexError(f"Unable to find {date} file within {url}")
@@ -309,27 +326,23 @@ class DwdMosmixValues(ScalarValuesCore):
         return df_urls["URL"].item()
 
 
-class DwdMosmixRequest(ScalarRequestCore):
+class DwdMosmixRequest(TimeseriesRequest):
     """Implementation of sites for MOSMIX mosmix sites"""
 
-    provider = Provider.DWD
-    kind = Kind.FORECAST
-
+    _provider = Provider.DWD
+    _kind = Kind.FORECAST
     _tz = Timezone.GERMANY
+    _dataset_base = DwdMosmixDataset
     _parameter_base = DwdMosmixParameter
-    _values = DwdMosmixValues
-
+    _unit_base = DwdMosmixUnit
     _resolution_type = ResolutionType.FIXED
     _resolution_base = Resolution  # use general Resolution for fixed Resolution
     _period_type = PeriodType.FIXED
-    _period_base = Period.FUTURE
-    _data_range = DataRange.FIXED
+    _period_base = DwdMosmixPeriod
     _has_datasets = True
     _unique_dataset = True
-    _has_tidy_data = False
-
-    _dataset_base = DwdMosmixDataset
-    _unit_tree = DwdMosmixUnit
+    _data_range = DataRange.FIXED
+    _values = DwdMosmixValues
 
     _url = "https://www.dwd.de/DE/leistungen/met_verfahren_mosmix/mosmix_stationskatalog.cfg?view=nasPublication"
 
@@ -343,14 +356,14 @@ class DwdMosmixRequest(ScalarRequestCore):
         return self.mosmix_type.name
 
     @classmethod
-    def _setup_discover_filter(cls, filter_):
+    def _setup_resolution_filter(cls, resolution):
         """
         Use SMALL and LARGE instead of resolution, which is fixed for Mosmix
 
-        :param filter_:
+        :param resolution:
         :return:
         """
-        return pd.Series(filter_, dtype=object).apply(
+        return pd.Series(resolution, dtype=object).apply(
             parse_enumeration_from_template, args=(cls._dataset_base,)
         ).tolist() or [*cls._dataset_base]
 
@@ -387,12 +400,14 @@ class DwdMosmixRequest(ScalarRequestCore):
 
     def __init__(
         self,
-        parameter: Optional[Tuple[Union[str, DwdMosmixParameter], ...]],
+        parameter: Optional[List[Union[str, DwdMosmixParameter, Parameter]]],
         mosmix_type: Union[str, DwdMosmixType],
         start_issue: Optional[Union[str, datetime, DwdForecastDate]] = DwdForecastDate.LATEST,
         end_issue: Optional[Union[str, datetime]] = None,
-        start_date: Optional[Union[str, datetime]] = None,
-        end_date: Optional[Union[str, datetime]] = None,
+        start_date: Optional[Union[str, datetime, pd.Timestamp]] = None,
+        end_date: Optional[Union[str, datetime, pd.Timestamp]] = None,
+        station_group: Optional[DwdMosmixStationGroup] = None,
+        settings: Optional[Settings] = None,
     ) -> None:
         """
 
@@ -405,12 +420,21 @@ class DwdMosmixRequest(ScalarRequestCore):
         """
         self.mosmix_type = parse_enumeration_from_template(mosmix_type, DwdMosmixType)
 
+        if self.mosmix_type == DwdMosmixType.SMALL:
+            self.station_group = DwdMosmixStationGroup.ALL_STATIONS
+        else:
+            self.station_group = (
+                parse_enumeration_from_template(station_group, DwdMosmixStationGroup)
+                or DwdMosmixStationGroup.SINGLE_STATIONS
+            )
+
         super().__init__(
             parameter=parameter,
             start_date=start_date,
             end_date=end_date,
             resolution=Resolution.HOURLY,
             period=Period.FUTURE,
+            settings=settings,
         )
 
         if not start_issue:
@@ -466,33 +490,23 @@ class DwdMosmixRequest(ScalarRequestCore):
 
         :return:
         """
-        payload = download_file(self._url, CacheExpiry.METAINDEX)
+        payload = download_file(self._url, self.settings, CacheExpiry.METAINDEX)
 
         df = pd.read_fwf(
             StringIO(payload.read().decode(encoding="latin-1")),
-            skiprows=4,
+            skiprows=2,
             skip_blank_lines=True,
-            colspecs=[
-                (0, 5),
-                (6, 11),
-                (12, 17),
-                (18, 22),
-                (23, 44),
-                (44, 51),
-                (51, 58),
-                (59, 64),
-                (65, 71),
-                (72, 76),
-            ],
+            # When inferring colspecs, make sure to look at the whole file. Otherwise,
+            # > this messes things up quite often and columns where strings get longer
+            # > towards the end of the data may get messed up and truncated.
+            # https://github.com/pandas-dev/pandas/issues/21970#issuecomment-480273621
+            colspecs="infer",
+            infer_nrows=np.Infinity,
             na_values=["----"],
             header=None,
             dtype="str",
         )
 
-        df = df[(df.iloc[:, 0] != "=====") & (df.iloc[:, 0] != "TABLE") & (df.iloc[:, 0] != "clu")]
-
-        df = df.iloc[:, [2, 3, 4, 5, 6, 7]]
-
         df.columns = [
             Columns.STATION_ID.value,
             Columns.ICAO_ID.value,
@@ -502,8 +516,9 @@ class DwdMosmixRequest(ScalarRequestCore):
             Columns.HEIGHT.value,
         ]
 
-        # Convert coordinates from degree minutes to decimal degrees
-        df[Columns.LATITUDE.value] = df[Columns.LATITUDE.value].astype(float).apply(convert_dm_to_dd)
-        df[Columns.LONGITUDE.value] = df[Columns.LONGITUDE.value].astype(float).apply(convert_dm_to_dd)
+        # The data as published by DWD has its `LAT` and `LON` columns in "Degrees Minutes" format.
+        # `convert_dm_to_dd` takes care of converting the coordinates to "Decimal Degrees".
+        df[Columns.LATITUDE.value] = convert_dm_to_dd(df[Columns.LATITUDE.value].astype(float))
+        df[Columns.LONGITUDE.value] = convert_dm_to_dd(df[Columns.LONGITUDE.value].astype(float))
 
         return df.reindex(columns=self._base_columns)
