@@ -1,16 +1,17 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2018-2021, earthobservations developers.
+# Copyright (C) 2018-2021, earthobservations developers.
 # Distributed under the MIT License. See LICENSE for more info.
 import logging
 from datetime import datetime, timezone
 from itertools import repeat
-from typing import Dict, List, Optional, Union
+from typing import List, Optional, Union
 
+import numpy as np
 import pandas as pd
 from pandas import Timedelta, Timestamp
 
-from wetterdienst.core.scalar.request import ScalarRequestCore
-from wetterdienst.core.scalar.values import ScalarValuesCore
+from wetterdienst.core.timeseries.request import TimeseriesRequest
+from wetterdienst.core.timeseries.values import TimeseriesValues
 from wetterdienst.metadata.columns import Columns
 from wetterdienst.metadata.datarange import DataRange
 from wetterdienst.metadata.kind import Kind
@@ -19,9 +20,6 @@ from wetterdienst.metadata.provider import Provider
 from wetterdienst.metadata.resolution import Resolution, ResolutionType
 from wetterdienst.metadata.timezone import Timezone
 from wetterdienst.provider.dwd.index import _create_file_index_for_dwd_server
-from wetterdienst.provider.dwd.metadata.column_names import DwdColumns
-from wetterdienst.provider.dwd.metadata.constants import DWDCDCBase
-from wetterdienst.provider.dwd.metadata.datetime import DatetimeFormat
 from wetterdienst.provider.dwd.observation.download import (
     download_climate_observations_data_parallel,
 )
@@ -34,10 +32,6 @@ from wetterdienst.provider.dwd.observation.metadata import (
     DwdObservationParameter,
     DwdObservationResolution,
 )
-from wetterdienst.provider.dwd.observation.metadata.field_types import (
-    DATE_PARAMETERS_IRREGULAR,
-    STRING_PARAMETERS,
-)
 from wetterdienst.provider.dwd.observation.metadata.period import DwdObservationPeriod
 from wetterdienst.provider.dwd.observation.metadata.resolution import (
     HIGH_RESOLUTIONS,
@@ -52,12 +46,13 @@ from wetterdienst.provider.dwd.observation.util.parameter import (
     check_dwd_observations_dataset,
 )
 from wetterdienst.provider.dwd.util import build_parameter_set_identifier
+from wetterdienst.settings import Settings
 from wetterdienst.util.enumeration import parse_enumeration_from_template
 
 log = logging.getLogger(__name__)
 
 
-class DwdObservationValues(ScalarValuesCore):
+class DwdObservationValues(TimeseriesValues):
     """
     The DWDObservationData class represents a request for
     observation data as provided by the DWD service.
@@ -66,11 +61,6 @@ class DwdObservationValues(ScalarValuesCore):
     _tz = Timezone.GERMANY
     _data_tz = Timezone.UTC
     _has_quality = True
-
-    _string_parameters = STRING_PARAMETERS
-    _irregular_parameters = ()
-    _date_parameters = DATE_PARAMETERS_IRREGULAR
-
     _resolution_type = ResolutionType.MULTI
     _resolution_base = DwdObservationResolution
     _period_type = PeriodType.MULTI
@@ -129,7 +119,7 @@ class DwdObservationValues(ScalarValuesCore):
 
         for period in self.sr.period:
             if self.sr.resolution in HIGH_RESOLUTIONS and period == Period.HISTORICAL:
-                date_ranges = self._get_historical_date_ranges(station_id, dataset)
+                date_ranges = self._get_historical_date_ranges(station_id, dataset, self.sr.stations.settings)
 
                 for date_range in date_ranges:
                     periods_and_date_ranges.append((period, date_range))
@@ -143,7 +133,7 @@ class DwdObservationValues(ScalarValuesCore):
                 dataset, self.sr.resolution, period, station_id, date_range
             )
 
-            log.info(f"Acquiring observations data for {parameter_identifier}.")
+            log.info(f"Acquiring observation data for {parameter_identifier}.")
 
             if not check_dwd_observations_dataset(dataset, self.sr.resolution, period):
                 log.info(f"Invalid combination {dataset.value}/" f"{self.sr.resolution.value}/{period} is skipped.")
@@ -151,7 +141,7 @@ class DwdObservationValues(ScalarValuesCore):
                 continue
 
             remote_files = create_file_list_for_climate_observations(
-                station_id, dataset, self.sr.resolution, period, date_range
+                station_id, dataset, self.sr.resolution, period, self.sr.stations.settings, date_range
             )
 
             if len(remote_files) == 0:
@@ -165,7 +155,7 @@ class DwdObservationValues(ScalarValuesCore):
                 log.info(f"No files found for {parameter_identifier}. Station will be skipped.")
                 continue
 
-            filenames_and_files = download_climate_observations_data_parallel(remote_files)
+            filenames_and_files = download_climate_observations_data_parallel(remote_files, self.sr.stations.settings)
 
             period_df = parse_climate_observations_data(filenames_and_files, dataset, self.sr.resolution, period)
 
@@ -179,18 +169,16 @@ class DwdObservationValues(ScalarValuesCore):
         # Filter out values which already are in the DataFrame
         parameter_df = parameter_df.drop_duplicates(subset=Columns.DATE.value)
 
-        # TODO: temporary fix -> reduce time steps before 2000 by 1 hour
-        #  for 1minute and 10minutes resolution data
         if not parameter_df.empty:
-            if self.sr.resolution in (Resolution.MINUTE_1, Resolution.MINUTE_10):
-                # Have to parse dates here although they should actually be parsed
+            if self.sr.resolution in (Resolution.MINUTE_1, Resolution.MINUTE_5, Resolution.MINUTE_10):
+                # Have to parse dates here, although they should actually be parsed
                 # later on in the core API
                 parameter_df[Columns.DATE.value] = pd.to_datetime(
-                    parameter_df[Columns.DATE.value], format=self._datetime_format
+                    parameter_df[Columns.DATE.value], infer_datetime_format=True
                 )
-                return self._fix_timestamps(parameter_df)
+                parameter_df = self._fix_timestamps(parameter_df)
 
-        return parameter_df
+        return self._tidy_up_df(parameter_df, dataset)
 
     @staticmethod
     def _fix_timestamps(df: pd.DataFrame) -> pd.DataFrame:
@@ -216,7 +204,6 @@ class DwdObservationValues(ScalarValuesCore):
         :param dataset: dataset enumeration
         :return: tidied DataFrame
         """
-
         droppable_columns = [
             # Hourly
             # Cloud type
@@ -225,12 +212,12 @@ class DwdObservationValues(ScalarValuesCore):
             DwdObservationParameter.HOURLY.CLOUD_TYPE.CLOUD_TYPE_LAYER3_ABBREVIATION.value,
             DwdObservationParameter.HOURLY.CLOUD_TYPE.CLOUD_TYPE_LAYER4_ABBREVIATION.value,
             # Cloudiness
-            DwdObservationParameter.HOURLY.CLOUDINESS.CLOUD_COVER_TOTAL_INDICATOR.value,
+            DwdObservationParameter.HOURLY.CLOUDINESS.CLOUD_COVER_TOTAL_INDEX.value,
             # Solar
             DwdObservationParameter.HOURLY.SOLAR.END_OF_INTERVAL.value,
             DwdObservationParameter.HOURLY.SOLAR.TRUE_LOCAL_TIME.value,
             # Visibility
-            DwdObservationParameter.HOURLY.VISIBILITY.VISIBILITY_RANGE_INDICATOR.value,
+            DwdObservationParameter.HOURLY.VISIBILITY.VISIBILITY_RANGE_INDEX.value,
             # Weather
             DwdObservationParameter.HOURLY.WEATHER_PHENOMENA.WEATHER_TEXT.value,
         ]
@@ -266,13 +253,14 @@ class DwdObservationValues(ScalarValuesCore):
                         pd.Series(repeat(quality_precipitation, 2)).explode(),
                     ]
                 )
-        elif resolution == Resolution.SUBDAILY and DwdObservationDataset.WIND_EXTREME:
+        elif resolution == Resolution.SUBDAILY and dataset == DwdObservationDataset.WIND_EXTREME:
             quality_fx_3 = df.pop("qn_8_3")
             quality_fx_6 = df.pop("qn_8_6")
             quality = pd.concat([quality_fx_3, quality_fx_6])
         else:
+            meta_columns = [Columns.DATE.value, Columns.STATION_ID.value]
             quality = df.pop(df.columns[2])
-            quality = pd.Series(repeat(quality, df.shape[1])).explode()
+            quality = pd.Series(repeat(quality, df.shape[1] - len(meta_columns))).explode()
 
         possible_id_vars = (
             Columns.STATION_ID.value,
@@ -288,7 +276,8 @@ class DwdObservationValues(ScalarValuesCore):
             var_name=Columns.PARAMETER.value,
             value_name=Columns.VALUE.value,
         )
-        df_tidy[Columns.QUALITY.value] = quality.reset_index(drop=True)
+        df_tidy[Columns.QUALITY.value] = quality.values
+        df_tidy.loc[df_tidy[Columns.VALUE.value].isna(), Columns.QUALITY.value] = np.nan
 
         return df_tidy
 
@@ -305,34 +294,9 @@ class DwdObservationValues(ScalarValuesCore):
             return series.dt.tz_localize(self.data_tz)
         return series
 
-    def _coerce_irregular_parameter(self, series: pd.Series) -> pd.Series:
-        """
-        Only one particular parameter is irregular which is the related to some
-        datetime found in solar.
-
-        :param series:
-        :return:
-        """
-        return pd.to_datetime(series, format=DatetimeFormat.YMDH_COLUMN_M.value)
-
-    def _create_humanized_parameters_mapping(self) -> Dict[str, str]:
-        """
-        Reduce the creation of parameter mapping of the massive amount of parameters
-        by specifying the resolution.
-
-        :return:
-        """
-        hpm = {}
-
-        for parameter in DwdObservationParameter[self.sr.resolution.name]:
-            try:
-                hpm[parameter.value] = parameter.name.lower()
-            except AttributeError:
-                pass
-
-        return hpm
-
-    def _get_historical_date_ranges(self, station_id: str, dataset: DwdObservationDataset) -> List[str]:
+    def _get_historical_date_ranges(
+        self, station_id: str, dataset: DwdObservationDataset, settings: Settings
+    ) -> List[str]:
         """
         Get particular files for historical data which for high resolution is
         released in data chunks e.g. decades or monthly chunks
@@ -341,10 +305,15 @@ class DwdObservationValues(ScalarValuesCore):
         :param dataset:
         :return:
         """
-        file_index = create_file_index_for_climate_observations(dataset, self.sr.resolution, Period.HISTORICAL)
+        file_index = create_file_index_for_climate_observations(
+            dataset, self.sr.resolution, Period.HISTORICAL, settings
+        )
 
         file_index = file_index[(file_index[Columns.STATION_ID.value] == station_id)]
 
+        if file_index.empty:
+            return []
+
         # The request interval may be None, if no start and end date
         # is given but rather the entire available data is queried.
         # In this case the interval should overlap with all files
@@ -353,7 +322,6 @@ class DwdObservationValues(ScalarValuesCore):
         if not interval:
             from_date_min = file_index[Columns.FROM_DATE.value].min()
             to_date_max = file_index[Columns.TO_DATE.value].max()
-
             interval = pd.Interval(from_date_min, to_date_max, closed="both")
 
         # Filter for from date and end date
@@ -365,30 +333,26 @@ class DwdObservationValues(ScalarValuesCore):
         return file_index_filtered[Columns.DATE_RANGE.value].tolist()
 
 
-class DwdObservationRequest(ScalarRequestCore):
+class DwdObservationRequest(TimeseriesRequest):
     """
     The DWDObservationStations class represents a request for
     a station list as provided by the DWD service.
     """
 
-    provider = Provider.DWD
-    kind = Kind.OBSERVATION
-
-    _values = DwdObservationValues
-    _parameter_base = DwdObservationParameter
+    _provider = Provider.DWD
+    _kind = Kind.OBSERVATION
     _tz = Timezone.GERMANY
-
+    _dataset_base = DwdObservationDataset
+    _parameter_base = DwdObservationParameter
+    _unit_base = DwdObservationUnit
     _resolution_type = ResolutionType.MULTI
     _resolution_base = DwdObservationResolution
     _period_type = PeriodType.MULTI
     _period_base = DwdObservationPeriod
-    _data_range = DataRange.FIXED
     _has_datasets = True
-    _has_tidy_data = False
     _unique_dataset = False
-    _dataset_base = DwdObservationDataset
-
-    _unit_tree = DwdObservationUnit
+    _data_range = DataRange.FIXED
+    _values = DwdObservationValues
 
     @property
     def _interval(self) -> Optional[pd.Interval]:
@@ -482,11 +446,15 @@ class DwdObservationRequest(ScalarRequestCore):
 
     def __init__(
         self,
-        parameter: Union[str, DwdObservationDataset],
+        parameter: Union[
+            Union[str, DwdObservationDataset, DwdObservationParameter],
+            List[Union[str, DwdObservationDataset, DwdObservationParameter]],
+        ],
         resolution: Union[str, Resolution, DwdObservationResolution],
         period: Optional[Union[str, Period, DwdObservationPeriod]] = None,
         start_date: Optional[Union[str, datetime, pd.Timestamp]] = None,
         end_date: Optional[Union[str, datetime, pd.Timestamp]] = None,
+        settings: Optional[Settings] = None,
     ):
         """
 
@@ -502,6 +470,7 @@ class DwdObservationRequest(ScalarRequestCore):
             period=period,
             start_date=start_date,
             end_date=end_date,
+            settings=settings,
         )
 
         if self.start_date and self.period:
@@ -535,7 +504,8 @@ class DwdObservationRequest(ScalarRequestCore):
             dataset=dataset,
             resolution=resolution,
             period=period,
-            cdc_base=DWDCDCBase.CLIMATE_OBSERVATIONS,
+            cdc_base="observations_germany/climate/",
+            settings=Settings.default(),
         )
 
         if language == "en":
@@ -545,9 +515,9 @@ class DwdObservationRequest(ScalarRequestCore):
         else:
             raise ValueError("Only language 'en' or 'de' supported")
 
-        file_index = file_index[file_index[DwdColumns.FILENAME.value].str.contains(file_prefix)]
+        file_index = file_index[file_index["filename"].str.contains(file_prefix)]
 
-        description_file_url = str(file_index[DwdColumns.FILENAME.value].tolist()[0])
+        description_file_url = str(file_index["filename"].tolist()[0])
         log.info(f"Acquiring field information from {description_file_url}")
 
         return read_description(description_file_url, language=language)
@@ -572,9 +542,9 @@ class DwdObservationRequest(ScalarRequestCore):
 
                     continue
 
-                df = create_meta_index_for_climate_observations(dataset, self.resolution, period)
+                df = create_meta_index_for_climate_observations(dataset, self.resolution, period, self.settings)
 
-                file_index = create_file_index_for_climate_observations(dataset, self.resolution, period)
+                file_index = create_file_index_for_climate_observations(dataset, self.resolution, period, self.settings)
 
                 df = df[df.loc[:, Columns.STATION_ID.value].isin(file_index[Columns.STATION_ID.value])]
 
@@ -588,6 +558,6 @@ class DwdObservationRequest(ScalarRequestCore):
         stations_df = stations_df.drop_duplicates(subset=Columns.STATION_ID.value, keep="first")
 
         if not stations_df.empty:
-            return stations_df.sort_values([Columns.STATION_ID.value], key=lambda x: x.astype(int))
+            return stations_df.sort_values(by=[Columns.STATION_ID.value], key=lambda x: x.astype(int))
 
         return stations_df
