@@ -3,7 +3,6 @@
 # Distributed under the MIT License. See LICENSE for more info.
 import os
 from typing import Optional
-from urllib.parse import urljoin
 
 import pandas as pd
 from dateparser import parse
@@ -11,8 +10,6 @@ from dateparser import parse
 from wetterdienst.metadata.extension import Extension
 from wetterdienst.metadata.period import Period
 from wetterdienst.metadata.resolution import Resolution
-from wetterdienst.provider.dwd.metadata.column_names import DwdColumns
-from wetterdienst.provider.dwd.metadata.constants import DWD_CDC_PATH, DWD_SERVER
 from wetterdienst.provider.dwd.metadata.datetime import DatetimeFormat
 from wetterdienst.provider.dwd.radar.metadata import (
     RADAR_PARAMETERS_COMPOSITES,
@@ -29,6 +26,7 @@ from wetterdienst.provider.dwd.radar.util import (
     RADOLAN_DT_PATTERN,
     get_date_from_filename,
 )
+from wetterdienst.settings import Settings
 from wetterdienst.util.cache import CacheExpiry
 from wetterdienst.util.network import list_remote_files_fsspec
 
@@ -52,6 +50,7 @@ def use_cache() -> int:  # pragma: no cover
 
 def create_fileindex_radar(
     parameter: DwdRadarParameter,
+    settings: Settings,
     site: Optional[DwdRadarSite] = None,
     fmt: Optional[DwdRadarDataFormat] = None,
     subset: Optional[DwdRadarDataSubset] = None,
@@ -64,6 +63,7 @@ def create_fileindex_radar(
     bin bufr or odim-hdf5 data. The file index is created for a single parameter.
 
     :param parameter:       The radar moment to request
+    :param settings:
     :param site:            Site/station if parameter is one of
                             RADAR_PARAMETERS_SITES
     :param fmt:          Data format (BINARY, BUFR, HDF5)
@@ -86,35 +86,35 @@ def create_fileindex_radar(
         period=period,
     )
 
-    url = urljoin(DWD_SERVER, parameter_path)
+    url = f"https://opendata.dwd.de/{parameter_path}"
 
-    files_server = list_remote_files_fsspec(url, ttl=CacheExpiry.NO_CACHE)
+    files_server = list_remote_files_fsspec(url, settings=settings, ttl=CacheExpiry.NO_CACHE)
 
-    files_server = pd.DataFrame(files_server, columns=[DwdColumns.FILENAME.value], dtype="str")
+    files_server = pd.DataFrame(files_server, columns=["filename"], dtype="str")
 
     # Some directories have both "---bin" and "---bufr" files within the same directory,
     # so we need to filter here by designated RadarDataFormat. Example:
     # https://opendata.dwd.de/weather/radar/sites/px/boo/
     if fmt is not None:
         if fmt == DwdRadarDataFormat.BINARY:
-            files_server = files_server[files_server[DwdColumns.FILENAME.value].str.contains("--bin")]
+            files_server = files_server[files_server["filename"].str.contains("--bin")]
         elif fmt == DwdRadarDataFormat.BUFR:
-            files_server = files_server[files_server[DwdColumns.FILENAME.value].str.contains("--buf")]
+            files_server = files_server[files_server["filename"].str.contains("--buf")]
 
     # Drop duplicates of files packed as .bz2, if not all files are .bz2
-    if not all(files_server[DwdColumns.FILENAME.value].str.endswith(".bz2")):
-        files_server = files_server[~files_server[DwdColumns.FILENAME.value].str.endswith(".bz2")]
+    if not all(files_server["filename"].str.endswith(".bz2")):
+        files_server = files_server[~files_server["filename"].str.endswith(".bz2")]
 
     # Decode datetime of file for filtering.
     if parse_datetime:
-        files_server[DwdColumns.DATETIME.value] = files_server[DwdColumns.FILENAME.value].apply(get_date_from_filename)
+        files_server["datetime"] = files_server["filename"].apply(get_date_from_filename)
 
         return files_server.dropna()
 
     return files_server
 
 
-def create_fileindex_radolan_cdc(resolution: Resolution, period: Period) -> pd.DataFrame:
+def create_fileindex_radolan_cdc(resolution: Resolution, period: Period, settings: Settings) -> pd.DataFrame:
     """
     Function used to create a file index for the RADOLAN_CDC product. The file index
     will include both recent as well as historical files. A datetime column is created
@@ -128,18 +128,16 @@ def create_fileindex_radolan_cdc(resolution: Resolution, period: Period) -> pd.D
     :return:                File index as DataFrame
     """
     file_index = create_fileindex_radar(
-        parameter=DwdRadarParameter.RADOLAN_CDC,
-        resolution=resolution,
-        period=period,
+        parameter=DwdRadarParameter.RADOLAN_CDC, resolution=resolution, period=period, settings=settings
     )
 
     file_index = file_index[
-        file_index[DwdColumns.FILENAME.value].str.contains("/bin/")
-        & file_index[DwdColumns.FILENAME.value].str.endswith((Extension.GZ.value, Extension.TAR_GZ.value))
+        file_index["filename"].str.contains("/bin/")
+        & file_index["filename"].str.endswith((Extension.GZ.value, Extension.TAR_GZ.value))
     ].copy()
 
     # Decode datetime of file for filtering.
-    file_index[DwdColumns.DATETIME.value] = file_index[DwdColumns.FILENAME.value].apply(
+    file_index["datetime"] = file_index["filename"].apply(
         lambda filename: parse(
             RADOLAN_DT_PATTERN.findall(filename)[0],
             date_formats=[DatetimeFormat.YM.value, DatetimeFormat.ymdhm.value],
@@ -191,9 +189,9 @@ def build_path_to_parameter(
             # See also page 4 on
             # https://opendata.dwd.de/climate_environment/CDC/help/RADOLAN/Unterstuetzungsdokumente/
             # Unterstuetzungsdokumente-Verwendung_von_RADOLAN-Produkten_im_ASCII-GIS-Rasterformat_in_GIS.pdf
-            return f"{DWD_CDC_PATH}/grids_germany/{resolution.value}/radolan/reproc/2017_002/bin"
+            return f"climate_environment/CDC/grids_germany/{resolution.value}/radolan/reproc/2017_002/bin"
         else:
-            return f"{DWD_CDC_PATH}/grids_germany/{resolution.value}/radolan/{period.value}/bin"
+            return f"climate_environment/CDC/grids_germany/{resolution.value}/radolan/{period.value}/bin"
 
     elif parameter in RADAR_PARAMETERS_COMPOSITES:
         return f"weather/radar/composit/{parameter.value}"
