@@ -6,17 +6,18 @@ import logging
 from datetime import datetime
 from enum import Enum
 from io import BytesIO
-from typing import Generator, Optional, Tuple, Union
+from typing import Generator, List, Optional, Tuple, Union
 
 import pandas as pd
 from pandas._libs.tslibs.offsets import YearEnd
 
-from wetterdienst.core.scalar.request import ScalarRequestCore
-from wetterdienst.core.scalar.values import ScalarValuesCore
+from wetterdienst.core.timeseries.request import TimeseriesRequest
+from wetterdienst.core.timeseries.values import TimeseriesValues
 from wetterdienst.exceptions import FailedDownload
 from wetterdienst.metadata.columns import Columns
 from wetterdienst.metadata.datarange import DataRange
 from wetterdienst.metadata.kind import Kind
+from wetterdienst.metadata.parameter import Parameter
 from wetterdienst.metadata.period import Period, PeriodType
 from wetterdienst.metadata.provider import Provider
 from wetterdienst.metadata.resolution import Resolution, ResolutionType
@@ -29,20 +30,19 @@ from wetterdienst.provider.eccc.observation.metadata.resolution import (
     EcccObservationResolution,
 )
 from wetterdienst.provider.eccc.observation.metadata.unit import EcccObservationUnit
+from wetterdienst.settings import Settings
 from wetterdienst.util.cache import CacheExpiry
 from wetterdienst.util.network import download_file
 
 log = logging.getLogger(__name__)
 
 
-class EcccObservationValues(ScalarValuesCore):
+class EcccObservationPeriod(Enum):
+    HISTORICAL = Period.HISTORICAL.value
 
-    _string_parameters = ()
-    _irregular_parameters = ()
-    _date_parameters = ()
 
+class EcccObservationValues(TimeseriesValues):
     _data_tz = Timezone.UTC
-
     _has_quality = True
 
     _base_url = (
@@ -75,14 +75,14 @@ class EcccObservationValues(ScalarValuesCore):
         """internal time step string for resolution"""
         return self._time_step_mapping.get(self.sr.stations.resolution)
 
-    def _tidy_up_df(self, df: pd.DataFrame, dataset) -> pd.DataFrame:
+    @staticmethod
+    def _tidy_up_df(df: pd.DataFrame) -> pd.DataFrame:
         """
         Tidy up dataframe pairwise by column 'DATE', 'Temp (°C)', 'Temp Flag', ...
 
         :param df: DataFrame with loaded data
         :return: tidied DataFrame
         """
-
         data = []
 
         columns = df.columns
@@ -115,17 +115,12 @@ class EcccObservationValues(ScalarValuesCore):
         """
         meta = self.sr.df[self.sr.df[Columns.STATION_ID.value] == station_id]
 
-        name, from_date, to_date = (
-            meta[
-                [
-                    Columns.NAME.value,
-                    Columns.FROM_DATE.value,
-                    Columns.TO_DATE.value,
-                ]
+        from_date, to_date = meta[
+            [
+                Columns.FROM_DATE.value,
+                Columns.TO_DATE.value,
             ]
-            .values.flatten()
-            .tolist()
-        )
+        ].values.flatten()
 
         # start and end year from station
         start_year = None if pd.isna(from_date) else from_date.year
@@ -148,7 +143,7 @@ class EcccObservationValues(ScalarValuesCore):
             for url in self._create_file_urls(station_id, start_year, end_year):
                 log.info(f"Acquiring file from {url}")
 
-                payload = download_file(url, CacheExpiry.NO_CACHE)
+                payload = download_file(url, self.sr.stations.settings, CacheExpiry.NO_CACHE)
 
                 df_temp = pd.read_csv(payload)
 
@@ -188,6 +183,12 @@ class EcccObservationValues(ScalarValuesCore):
 
         df[Columns.STATION_ID.value] = station_id
 
+        df = self._tidy_up_df(df)
+
+        mask = df[Columns.VALUE.value].map(str).str.startswith("<")
+        df.loc[mask, Columns.VALUE.value] = df.loc[mask, Columns.VALUE.value].str[1:]
+        df[Columns.QUALITY.value] = pd.NA
+
         return df
 
     def _create_file_urls(self, station_id: str, start_year: int, end_year: int) -> Generator[str, None, None]:
@@ -217,7 +218,7 @@ class EcccObservationValues(ScalarValuesCore):
             yield url
 
 
-class EcccObservationRequest(ScalarRequestCore):
+class EcccObservationRequest(TimeseriesRequest):
     """
     Download weather data from Environment and Climate Change Canada (ECCC).
     - https://www.canada.ca/en/environment-climate-change.html
@@ -228,24 +229,19 @@ class EcccObservationRequest(ScalarRequestCore):
 
     """
 
-    provider = Provider.ECCC
-    kind = Kind.OBSERVATION
-
+    _provider = Provider.ECCC
+    _kind = Kind.OBSERVATION
     _tz = Timezone.UTC
-
-    _resolution_base = EcccObservationResolution
-    _resolution_type = ResolutionType.MULTI
-    _period_type = PeriodType.FIXED
-    _period_base = Period.HISTORICAL
-    _parameter_base = EcccObservationParameter  # replace with parameter enumeration
-    _data_range = DataRange.LOOSELY
-    _has_datasets = True
     _dataset_base = EcccObservationDataset
+    _parameter_base = EcccObservationParameter  # replace with parameter enumeration
+    _unit_base = EcccObservationUnit
+    _resolution_type = ResolutionType.MULTI
+    _resolution_base = EcccObservationResolution
+    _period_type = PeriodType.FIXED
+    _period_base = EcccObservationPeriod
+    _has_datasets = True
     _unique_dataset = True
-    _has_tidy_data = False
-
-    _unit_tree = EcccObservationUnit
-
+    _data_range = DataRange.FIXED
     _values = EcccObservationValues
 
     @property
@@ -293,10 +289,11 @@ class EcccObservationRequest(ScalarRequestCore):
 
     def __init__(
         self,
-        parameter: Tuple[Union[str, EcccObservationParameter]],
-        resolution: Union[EcccObservationResolution, Resolution],
-        start_date: Optional[datetime] = None,
-        end_date: Optional[datetime] = None,
+        parameter: List[Union[str, EcccObservationParameter, Parameter]],
+        resolution: Union[str, EcccObservationResolution, Resolution],
+        start_date: Optional[Union[str, datetime, pd.Timestamp]] = None,
+        end_date: Optional[Union[str, datetime, pd.Timestamp]] = None,
+        settings: Optional[Settings] = None,
     ):
         """
 
@@ -311,6 +308,7 @@ class EcccObservationRequest(ScalarRequestCore):
             period=Period.HISTORICAL,
             start_date=start_date,
             end_date=end_date,
+            settings=settings,
         )
 
     def _all(self) -> pd.DataFrame:
@@ -332,8 +330,7 @@ class EcccObservationRequest(ScalarRequestCore):
 
         return df
 
-    @staticmethod
-    def _download_stations() -> Tuple[BytesIO, int]:
+    def _download_stations(self) -> Tuple[BytesIO, int]:
         """
         Download station list from ECCC FTP server.
 
@@ -349,14 +346,14 @@ class EcccObservationRequest(ScalarRequestCore):
         payload = None
         source = None
         try:
-            payload = download_file(gdrive_url, CacheExpiry.METAINDEX)
+            payload = download_file(gdrive_url, self.settings, CacheExpiry.METAINDEX)
             source = 0
         except Exception:
             log.exception(f"Unable to access Google drive server at {gdrive_url}")
 
             # Fall back to different source.
             try:
-                response = download_file(http_url, CacheExpiry.METAINDEX)
+                response = download_file(http_url, self.settings, CacheExpiry.METAINDEX)
                 with gzip.open(response, mode="rb") as f:
                     payload = BytesIO(f.read())
                 source = 1
@@ -364,6 +361,6 @@ class EcccObservationRequest(ScalarRequestCore):
                 log.exception(f"Unable to access HTTP server at {http_url}")
 
         if payload is None:
-            raise FailedDownload("Unable to acquire ECCC stations_result list")
+            raise FailedDownload("Unable to acquire ECCC stations list")
 
         return payload, source
